package uoc.tfc.alarm;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import uoc.tfc.clientbs.AlarmActivity;
import uoc.tfc.clientbs.R;
import uoc.tfc.database.Database;
import uoc.tfc.entities.TakeItem;
import uoc.tfc.ui.UIUtils;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.PowerManager;
import android.util.Log;
import android.view.WindowManager;

/**
 * Broadcast receiver for managing alarms.
 * 
 * @author omnizeus
 *
 */
public class AlarmManagerBroadcastReceiver extends BroadcastReceiver {

	// Alarm label.
	final public static String ONE_TIME = "onetime";

	// Database elements
	private Database db;

	// UI utilites.
	UIUtils uiutils;

	/**
	 * On Receive event.
	 */
	@Override
	public void onReceive(Context context, Intent intent) {

		PowerManager pm = null;
		PowerManager.WakeLock wl = null;
		String alarmTag;
		String whereami = ":onReceive";
		try {

			/**
			 * Label for alarm
			 */
			alarmTag = context.getPackageName() + "alarm";

			/**
			 * PowerManager & WakeLock to wake the device if necessary.
			 */
			pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
			wl = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, alarmTag);

			/**
			 * Acquire the lock
			 */
			wl.acquire();

			Log.i(this.getClass().getName() + whereami,
					"Inicio: ONE_TIME:"
							+ intent.getExtras().getBoolean(ONE_TIME)
							+ "; requestCode:"
							+ intent.getExtras().getInt("requestCode"));
			Log.i(this.getClass().getName() + whereami, " - WakeLock adquirido");
			Log.i(this.getClass().getName() + whereami, " - Proceso de alarma");

			/**
			 * Prepare database
			 */
			db = new Database(context);

			/**
			 * Get only one take item.
			 */
			TakeItem takeItem = db.GetOneTakeItem(intent.getExtras().getInt(
					"requestCode"));

			if (takeItem != null) {

				uiutils = new UIUtils(context);

				/**
				 * A notification.
				 */
				uiutils.setNotification(context, "ALARMA",
						"ATENCIÓ, ATENCIÓ, ATENCIÓ", R.raw.button11);

				/**
				 * Prepare the intent (Alarm Activity).
				 */
				Intent i = new Intent();
				i.setClass(context, AlarmActivity.class);
				i.setFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
						| Intent.FLAG_ACTIVITY_NEW_TASK
						| Intent.FLAG_ACTIVITY_CLEAR_TOP
						| Intent.FLAG_ACTIVITY_SINGLE_TOP
						| Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);

				/**
				 * Put parameters in Alarm intent.
				 */
				Bundle extras = new Bundle();
				extras.putInt("requestCode",
						intent.getExtras().getInt("requestCode"));

				// More parameters (not necessary).
				/*
				 * extras.putLong("id", takeItem.getId()); extras.putInt("toma",
				 * takeItem.getToma()); extras.putString("date",
				 * takeItem.getDate()); extras.putString("product",
				 * takeItem.getProduct()); extras.putInt("quantity",
				 * takeItem.getQuantity());
				 */

				i.putExtras(extras);

				Log.i(this.getClass().getName() + whereami, extras.toString());

				context.startActivity(i);

			} else {
				
				Log.i(this.getClass().getName() + whereami, "Nothing to do!");
			}
			

		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami,
					"Exception (During WakeLock)", e);
		} finally {
			try {
				/**
				 * Release the lock
				 */
				if (wl != null) {
					wl.release();
					Log.i(this.getClass().getName() + whereami,
							" - WakeLock released.");
				}
			} catch (Exception e) {
				Log.e(this.getClass().getName() + whereami,
						"Exception (Releasing WakeLock)", e);
			}
		}
	}

	/**
	 * Set a new Alarm with requestCode.
	 * 
	 * @param context
	 * @param requestCode
	 * @param hh
	 * @param mm
	 * @param ss
	 */
	public void SetAlarm(Context context, int requestCode, int hh, int mm) {
		String whereami = ":SetAlarm";
		AlarmManager am = null;
		Intent intent = null;
		Log.d(this.getClass().getName() + ":" + "SetAlarm", "requestCode: "
				+ requestCode + "; " + hh + ":" + mm);

		try {
			/**
			 * Alarm Manager & Intent (broadcast receiver).
			 */
			am = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
			intent = new Intent(context, AlarmManagerBroadcastReceiver.class);

			/**Add multiple parameters
			 * 
			 */
			Bundle extras = new Bundle();
			extras.putBoolean(ONE_TIME, Boolean.FALSE);
			extras.putInt("requestCode", requestCode);
			intent.putExtras(extras);

			/**
			 * Dates
			 */
			Date dat = new Date();
			Calendar cal_alarm = Calendar.getInstance();
			Calendar cal_now = Calendar.getInstance();
			cal_now.setTime(dat);
			cal_alarm.setTime(dat);
			cal_alarm.set(Calendar.HOUR_OF_DAY, hh);
			cal_alarm.set(Calendar.MINUTE, mm);
			cal_alarm.set(Calendar.SECOND, 0);
			if (cal_alarm.before(cal_now)) {// if its in the past increment
				cal_alarm.add(Calendar.DATE, 1);
			}

			/**
			 * Formatting date
			 */
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			String cur_cal_text = sdf.format(cal_now.getTime());
			String cal_text = sdf.format(cal_alarm.getTime());

			Log.d(this.getClass().getName() + ":" + "SetAlarm", ">>> "
					+ cur_cal_text + "; " + cal_text);

			PendingIntent pi = PendingIntent.getBroadcast(context, requestCode,
					intent, 0);
			 
			/**
			 * Repeat alarm after 30 seconds
			 */
			am.setRepeating(AlarmManager.RTC_WAKEUP,
					cal_alarm.getTimeInMillis(), 1000 * 30, pi);

		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * Cancel all stored alarms.
	 * 
	 * @param context
	 * @param requestCode
	 */
	public void CancelAlarm(Context context, int requestCode) {
		String whereami = ":CancelAlarm";
		try {
			Intent intent = new Intent(context, AlarmManagerBroadcastReceiver.class);
			PendingIntent sender = PendingIntent.getBroadcast(context, requestCode,
					intent, 0);
			AlarmManager alarmManager = (AlarmManager) context
					.getSystemService(Context.ALARM_SERVICE);
			alarmManager.cancel(sender);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * 
	 * 
	 * @param context
	 * @param requestCode
	 */
	public void setOnetimeTimer(Context context, int requestCode) {
		String whereami = ":setOnetimeTimer";
		try {
			AlarmManager am = (AlarmManager) context
					.getSystemService(Context.ALARM_SERVICE);
			Intent intent = new Intent(context, AlarmManagerBroadcastReceiver.class);
			intent.putExtra(ONE_TIME, Boolean.TRUE);
			PendingIntent pi = PendingIntent.getBroadcast(context, requestCode,
					intent, 0);
			am.setRepeating(AlarmManager.RTC_WAKEUP, System.currentTimeMillis(),
					1000 * 10, pi);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}

	}

}