package uoc.tfc.database;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * Class for managing local database setup and update operations.
 * 
 * @author omnizeus
 *
 */
public class DatabaseHelper extends SQLiteOpenHelper {

	private static DatabaseHelper sInstance;

	private static final String DATABASE_NAME = "ClientBS";
	private static final int DATABASE_VERSION = 2;

	/**
	 * Initialize DatabaseHelper with only one instance for entire App.
	 * 
	 * @param context
	 * @return
	 */
	public static DatabaseHelper getInstance(Context context) {
		if (sInstance == null) {
			sInstance = new DatabaseHelper(context.getApplicationContext());
		}
		return sInstance;
	}

	/**
	 * Constructor.
	 * 
	 * @param context
	 */
	public DatabaseHelper(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	//
	/**
	 * Database event: called during creation of the database. Create all
	 * database structure.
	 */
	@Override
	public void onCreate(SQLiteDatabase database) {
		String whereami = ":onCreate";
		try {
			
			// Create the user table (with service access data)
			database.execSQL("CREATE TABLE IF NOT EXISTS user ("
					+ "nickname VARCHAR(50)," + "password VARCHAR(50),"
					+ "idpatientph VARCHAR(50),"
					+ "clientbspassword VARCHAR(16)" + ");");
			Log.i(this.getClass().getName() + whereami, "Created [user] table.");

			// Create the takelist table
			database.execSQL("CREATE TABLE IF NOT EXISTS takelist ("
					+ "id INTEGER PRIMARY KEY," + "date VARCHAR(50),"
					+ "toma INTEGER," + "product VARCHAR(250),"
					+ "quantity INTEGER," + "status INTEGER DEFAULT 0" +
					");");

			Log.i(this.getClass().getName() + whereami,
					"Created [takelist] table.");

			// Create the alarm table
			database.execSQL("CREATE TABLE IF NOT EXISTS horarylist ("
					+ "toma INTEGER," + "label VARCHAR(100)," + "hour INTEGER,"
					+ "minute INTEGER" + ");");

			Log.i(this.getClass().getName() + whereami,
					"Created [horarylist] table.");

			
			database.execSQL("DELETE FROM horarylist;");
			
			// Add default entries
			database.execSQL("INSERT INTO horarylist (toma, label, hour, minute) VALUES (1, 'MATÍ', 7, 15);");
			database.execSQL("INSERT INTO horarylist (toma, label, hour, minute) VALUES (2, 'MIGDIA', 12, 15);");
			database.execSQL("INSERT INTO horarylist (toma, label, hour, minute) VALUES (3, 'TARDA', 17, 15);");
			database.execSQL("INSERT INTO horarylist (toma, label, hour, minute) VALUES (4, 'NIT', 22, 15);");

			Log.i(this.getClass().getName() + whereami,
					"Added default values for [horarylist] table.");
			
		} catch (SQLiteException e) {
			Log.e(this.getClass().getName() + whereami, "SQLiteException", e);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * Database event: on upgrade to a newer version. Regenerate all tables.
	 */
	@Override
	public void onUpgrade(SQLiteDatabase database, int oldVersion,
			int newVersion) {
		String whereami = ":onUpgrade";
		try {
			Log.i(this.getClass().getName() + whereami,
					"Upgrading database from version " + oldVersion + " to "
							+ newVersion + ", which will destroy all old data:");
			// Drops the existent tables
			clearDatabase(database);
			// Creates a new table structure
			onCreate(database);
			database.close();
		} catch (SQLiteException e) {
			Log.e(this.getClass().getName() + whereami, "SQLiteException", e);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}
	
	/**
	 * Auxiliary method for remove all tables.
	 * @param database
	 * @return
	 */
	private boolean clearDatabase(SQLiteDatabase database) {
		String whereami = ":clearDatabase";
		try {
			database.execSQL("DROP TABLE IF EXISTS takelist;");
			database.execSQL("DROP TABLE IF EXISTS horarylist;");
			database.execSQL("DROP TABLE IF EXISTS user;");			
		} catch (SQLiteException e) {
			Log.e(this.getClass().getName() + whereami, "SQLiteException", e);
			return false;
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
			return false;
		}
		return true;
	}

}